<?php
/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Filename:    we_baseElement.inc.php
 *
 * Function:    Utility class that implements basic html elements operations
 *
 * Description: Provides functions for creating html tags
 */
class we_html_baseElement{
	const SELF_CLOSE = 'selfclose';

	protected $tag_name = '';
	protected $need_end_tag = true;
	protected $attribs = ['style' => []];
	protected $content = '';

	/**
	 * Constructor
	 * @param		$attribs								array
	 * @param		$content								string
	 *
	 */
	public function __construct($tagname, $need_end_tag = true, array $attribs = [], $content = ''){
		$this->tag_name = $tagname;
		$this->need_end_tag = $need_end_tag;
		$this->setAttributes($attribs);
		$this->content = $content;
	}

	/**
	 * Function returns copy of object
	 *
	 * @return     we_html_baseElement
	 */
	protected function copy(){
		return clone $this;
	}

	/*
	 * Function sets element attributes
	 *
	 * @param		$attribs								array
	 *
	 * @return		void
	 */

	public function setAttributes(array $attribs){
		if(empty($attribs)){
			return;
		}
		foreach($attribs as $k => $v){
			if($k){
				$this->setAttribute($k, $v);
			}
		}
	}

	/**
	 * Function sets element attribute
	 *
	 * @param		$attrib_name							string
	 * @param		$attrib_value							string
	 *
	 * @return		void
	 */
	public function setAttribute($attrib_name, $attrib_value){
		switch($attrib_name){
			case 'style':
				$vals = is_array($attrib_value) ? $attrib_value : explode(';', rtrim($attrib_value, '; '));
				foreach($vals as $val){
					$val = trim($val);
					if($val){
						list($k, $v) = explode(':', $val);
						$this->setStyle(trim($k), trim($v));
					}
				}
				break;
			case 'valign':
				$this->setStyle('vertical-align', $attrib_value);
				break;
			case 'height':
			case 'width':
				$this->setStyle($attrib_name, $attrib_value . (is_numeric($attrib_value) ? 'px' : ''));
				break;
			case 'border':
				$this->setStyle('border-width', $attrib_value . (is_numeric($attrib_value) ? 'px' : ''));
				break;
			case 'bgcolor':
				$this->setStyle('background-color', $attrib_value);
				break;
			case 'background':
				$this->setStyle('background-image', 'url(' . $attrib_value . ')');
				break;
			case 'disabled':
			case 'multiple':
			case 'noshade':
			case 'nowrap':
			case 'readonly':
			case 'checked':
			case 'selected':
				if($attrib_value){
					$this->attribs[$attrib_name] = $attrib_name;
				}
				break;
			default:
				$this->attribs[$attrib_name] = $attrib_value;
		}
	}

	public function setStyle($type, $val){
		$this->attribs['style'][trim($type)] = trim($val);
	}

	public function hasStyle($type){
		return isset($this->attribs['style'][trim($type)]);
	}

	/**
	 * Function gets element attribute
	 *
	 * @param		$attrib_name							string
	 *
	 * @return		string
	 */
	public function getAttribute($attrib_name){
		return isset($this->attribs[$attrib_name]) ? $this->attribs[$attrib_name] : '';
	}

	/**
	 * The function generate HTML code for the tag
	 *
	 * @param		we_html_baseElement $object
	 *
	 * @return		string
	 */
	public static function getHtmlCode(we_html_baseElement $object){
		return $object->getHTML();
	}

	public static function getAttribsAsString(array $attr){
		$out = '';
		foreach($attr as $k => $v){
			switch($k){
				case 'style':
					if($v){
						$out .= ' ' . $k . '="';
						foreach($v as $kk => $vv){
							$out .= $kk . ':' . $vv . ';';
						}
						$out .= '"';
					}
					break;
				case 'SCRIPTABLE'://FIXME: remove
				case 'MAYSCRIPT':
					$out .= ' ' . $k;
					break;
				default:
					if($v !== ''){
						if(is_array($v)){
							t_e($v);
						}
						$out .= ' ' . $k . '="' . $v . '"';
					} else {
						//no empty attribs
						//$out .= ' ' . $k . '=""';
					}
					break;
			}
		}
		return $out;
	}

	public function getHTML(){
		if(is_array($this->content)){
			t_e($this);
		}
		return '<' . $this->tag_name . self::getAttribsAsString($this->attribs) .
			($this->need_end_tag === self::SELF_CLOSE ? '/>' : '>' .
			$this->content .
			($this->need_end_tag ? '</' . $this->tag_name . '>' : '')
			);
	}

	public function getContent(){
		return $this->content;
	}

}
